// ======================= MSP430F1xxx S/W   α׷ =================
//  CPU : MSP430F4250
//     ¥ : 2008-
//      : Ver1.0
//    : www.cpuplaza.com
// Ϸ : mspgcc+Dev-C++
// [ ]  : ACLK : 32.768Khz
//            MCLK : DCO(7.995392MHz)
//            SMCLK: DCO(7.995392MHz)
// ----------------------------------------------------------------------------
// ⺻ 
#include <io.h>
#include <stdio.h>
#include <signal.h>
#include <iomacros.h>

//  
//  
#define RXD	0x02						// RXD on P1.1
#define TXD	0x01						// TXD on P1.0

// Լ 
void cpu_setup(void);						// CPU ʱȭ

void com_set(long baud);					//  
void rx_ready(void);						//  غ
void putchar_0(unsigned char datac);				//  1Ʈ ۽
void puts_0(unsigned char dbuf[]);				//  ڿ 

//  
volatile unsigned int Bitime_5 = 0;				// (1 /Bps) / 2
volatile unsigned int Bitime = 0;				// 1 / Bps

volatile unsigned int TXData;					// ۽ 
volatile unsigned int RXData;					//  
volatile unsigned char TXBitCnt;				// ۽  ī
volatile unsigned char RXBitCnt;				//   ī

// ===============================  α׷ ==============================
// [μ]: void
// []: 115200Bps,N,8,1 :: Half Duplex
//         P1.1 : TXD
//         P2.2 : RXD
// ---------------------------------------------------------------------------
int main(void)
{
	cpu_setup();						// cpuʱȭ
	com_set(115200);					//  
	_EINT();						// Enable interrupts

	puts_0("serial test program\r\n");			// string tx
	rx_ready();						//  غ
	while(1){
		if((CCTL0 & CCIE) == 0){			//  Ϸ
			putchar_0(RXData);			// ۽
			rx_ready();				//  غ			
		}
	}
}

// ===================================== CPUʱȭ ============================
// [μ] void
// ----------------------------------------------------------------------------
void cpu_setup(void)
{
unsigned dly;
	
	WDTCTL = WDTPW + WDTHOLD;				// Init watchdog timer(stop)
	for(dly = 0; dly < 0x4000U; dly++) ;			// XTAL  ð 

	SCFI0 |= FN_4;						// Set DCO operating range
	SCFQCTL = 121;						// (121+1) x 32768(ACLK) x 2 = 7.99 Mhz
	FLL_CTL0 = DCOPLUS + XCAP18PF;				// DCO+ set so freq = xtal x D x N+1
}

// ================================   =================================
// [μ] int baud
// Item1: Ʈ (1200,2400,4800,9600,19200,57600,115200Bps)
// [] DCO Freq = 7.995352MHz 
// ----------------------------------------------------------------------------
void com_set(long baud)
{
	TACTL = TASSEL_2+TACLR+MC_2;				// Clear TAR, Source = SMCLK, Continous mode
	CCTL0 = OUT;						// TXD Idle as Mark 
	P1SEL = TXD + RXD;					// P1.0/1 TA0 for TXD/RXD function
	P1DIR = TXD;						// TXD output on P1
	switch(baud){
		case 1200 : Bitime_5 = 3331;			// 416.617us bit length 
			    Bitime = 6662;			// 833.234us(1200Bps)
			    break;		
		case 2400 : Bitime_5 = 1665;			// 208.308us bit length 
			    Bitime = 3331;			// 416.617us(2400Bps)
			    break;		
		case 4800 : Bitime_5 = 832;			// 104.122us bit length 
			    Bitime = 1665;			// 208.245us(4800Bps)
			    break;		
		case 9600 : Bitime_5 = 416;			// 52.03us bit length 
			    Bitime = 832;			// 104.122us(9600Bps)
			    break;		
		case 19200: Bitime_5 = 208;			// 26.015us bit length 
			    Bitime = 416;			// 52.03us(19200Bps)
			    break;		
		case 38400: Bitime_5 = 104;			// 13.007us bit length 
			    Bitime = 208;			// 26.015us(38400Bps)
			    break;		
		case 57600: Bitime_5 = 69;			// 8.63us bit length 
			    Bitime = 138;			// 17.26us(57600Bps)
			    break;		
		case 115200:Bitime_5 = 34;			// 4.252us bit length 
			    Bitime = 69;			// 8.63us(115200Bps)
			    break;		
	}
}

// =================================  غ ================================
// [μ] void
// ----------------------------------------------------------------------------
void rx_ready(void)
{
	RXBitCnt = 8;						// Load Bit counter
	CCTL0 = SCS+CCIS0+OUTMOD0+CM1+CAP+CCIE;			// Sync, Neg Edge, Capture          
}

// =============================== 1Ʈ ۽ ===============================
// [μ] unsigned char datac
// Item1:  (0 - ffh)
// ----------------------------------------------------------------------------
void putchar_0(unsigned char datac)
{
	TXBitCnt = 10;						// Load Bit counter, 8data + ST/SP
	CCR0 = TAR;						// Current state of TA counter
	CCR0 += Bitime;						// Some time till first bit
	TXData = (unsigned int)datac | 0x100;			// Add mark stop bit to TXData 
	TXData = TXData << 1;					// Add space start bit
	CCTL0 = OUTMOD0+CCIE;					// TXD = mark = idle
	while(CCTL0 & CCIE);					// Wait for TX completion
}

// ================================ Ʈ ۽ ===============================
// [μ] unsigned char dbuf
// Item1:   ڿ
// ----------------------------------------------------------------------------
void puts_0(unsigned char dbuf[])
{
unsigned char ch;
int i = 0;

	ch = dbuf[i++];
	while(ch != 0){
		putchar_0(ch);					// 1 send
		ch = dbuf[i++];
	}
}

// ============================ Timer A0 interrupt ============================
// [μ] void
// ----------------------------------------------------------------------------
interrupt (TIMERA0_VECTOR)Timer_A(void)
{
	CCR0 += Bitime;						// Add Offset to CCR0
// RX
	if(CCTL0 & CCIS0){					// RX on CCI0B?
		if(CCTL0 & CAP){				// Capture mode = start bit edge 
			CCTL0 &= ~ CAP;				// Switch from capture to compare mode 
			CCR0 += Bitime_5;
		}
		else{
			RXData = RXData >> 1;           
			if(CCTL0 & SCCI) RXData |= 0x80;	// Get bit waiting in receive latch
			RXBitCnt --;				// All bits RXed?
			if (RXBitCnt == 0){
				CCTL0 &= ~ CCIE;		// All bits RXed, disable interrupt
//				_BIC_SR_IRQ(CPUOFF);		// Clear LPM0 bits from 0(SR)	
			}
		}
	}
// TX
	else{
		if(TXBitCnt == 0) CCTL0 &= ~ CCIE;		// All bits TXed, disable interrupt
		else{
			CCTL0 |=  OUTMOD2;			// TX Space
			if(TXData & 0x01) CCTL0 &= ~ OUTMOD2;	// TX Mark
			TXData = TXData >> 1;
			TXBitCnt --;
		}
	}
}
